function wormfinder_ui(ifn,ofn,pars)
global skui_data
if isempty(skui_data)
  skui_data={};
end
f=figure;

if nargin<3
  pars.sigperp=8;
  pars.thresh=.25;
end
if ~isfield(pars,'sigperp')
  pars.sigperp=8;
end
if ~isfield(pars,'thresh')
  pars.thresh=.25;
end
if ~isfield(pars,'siglong')
  pars.siglong = [];
end
if ~isfield(pars,'lambda')
  pars.lambda=[];
end

ss=pars.sigperp;
if isempty(pars.siglong)
  ss=[ss ss(1)];
else
  ss=[ss pars.siglong];
end
if isempty(pars.lambda)
  ss=[ss 2.5*ss(1)];
else
  ss=[ss pars.lambda];
end
skui_data{f}.shownscale=ss;
  
if ~ischar(pars.sigperp)
  pars.sigperp=sprintf('%.2f',pars.sigperp);
end
if ~ischar(pars.siglong)
  pars.siglong=sprintf('%.2f',pars.siglong);
end
if ~ischar(pars.lambda)
  pars.lambda=sprintf('%.2f',pars.lambda);
end
if ~ischar(pars.thresh)
  pars.thresh=sprintf('%.2f',pars.thresh);
end

if length(skui_data)<f
  skui_data{f}.N=0;
end
if isempty(skui_data{f})
  skui_data{f}.N=0;
end
if ~isfield(skui_data{f},'N')
  skui_data{f}.N=0;
end

skui_data{f}.busy=0;

try
  set(f,'doublebuf','on');
catch
  set(f,'doublebuf','off');
end

set(f,'position',[200 100 720 640]);
set(f,'menubar','none');
ax0=axes('position',[0 0 1 1]); axis([0 720 0 640]); axis off

ax1=axes('units','pix','position',[50 140 640 480]);

CDX=18; RDY=5; RDY2=10;

h0 = ui_push('Save results',@skui_saveres,[15 12 100 25],[.7 .7 1],...
    'Save all detection results to a file.');
h = ui_push('Load results',@skui_loadmat,ui_above(h0,RDY),h0,...
    'Load results of a previous Wormfinder run.','userdata','always');
h1 = ui_push('Load image(s)',@skui_loadpng,ui_above(h,RDY2),h,...
    'Load one image or a series of images.','userdata','always');

axes(ax0);
h6 =  ui_edit('siglong',pars.siglong,@skui_siglong,ui_right(h0,CDX+14,40),'Longitudinal radius of Gabor patch (in pixels). Leave blank to reuse perpendicular radius.');
ui_figlabel(h6,'\sigma_{||}:');
h = ui_edit('sigperp',pars.sigperp,@skui_sigperp,ui_above(h6,RDY),'Perpendicular radius of Gabor patch (in pixels). (Set to expected approximate width of worm.)');
ui_figlabel(h,'\sigma_{\perp}:');
h6a = ui_edit('lambda',pars.lambda,@skui_lambda,ui_right(h6,CDX+38,40),'Wavelength of Gabor patch (in pixels). Leave blank to use perpendicular radius (x 2.5).');
ui_figlabel(h6a,'\lambda:');
h = ui_edit('thresh',pars.thresh,[],ui_above(h6a,RDY),'Threshold for termination (fraction of maximum intensity).');
ui_figlabel(h,'Thresh.:');
axes(ax1);

h2 = ui_push('Det. all',@skui_detectall,ui_right(h6a,CDX,60),[1 1 .7],'Detect worms in all images in which no worm has been detected yet.');
h  = ui_push('Detect',@skui_detect,ui_above(h2,RDY),h2,'Detect worm in this image.');


h3 = ui_radio('deint','+De-interlace',ui_right(h1,CDX/2,90),[.8 .8 1],...
    'If set, loaded images will be de-interlaced.','userdata','always');

h = ui_push('|<',@skui_first,ui_right(h3,CDX*1.5,25),[.7 .7 .7],'Go to first image.');
h = ui_push('<<',@skui_prev,ui_right(h,RDY),h,'Go to previous image.');
h4 = ui_edit('frameno','000/000',@skui_frameno,ui_right(h,RDY,70),...
    'Currently shown image and total number of images. Edit to jump to another image.');
h = ui_push('>>',@skui_next,ui_right(h4,RDY,25),h,'Go to next image.');
h10 = ui_push('>|',@skui_last,ui_right(h,RDY),h,'Go to last image.');


h7 = ui_radio('onlight','+On light',ui_right(h2,CDX,70),[.7 .9 .7],...
    'If set, detect animals on a light background.', ...
    @skui_lightdark);
h = ui_radio('ondark','On dark',ui_above(h7,RDY),h7,...
    'If set, detect animals on a dark background.', ...
    @skui_lightdark);

h8 = ui_radio('softmax','Softmax',ui_right(h7,CDX/2),h7,'If set, use weighted average direction; otherwise, use max direction.', ...
    @skui_invalidateimax);
h8a = ui_radio('cleanbg','+Clean b/g',ui_above(h8,RDY),h7,...
    'If set, subtract median background from stack of images before detection.', ...
    @skui_invalidateimax);

skui_data{f}.showwhat = 1; % BG was [.85 .65 1].
h11=uicontrol('string','Image|W/o b/g|Dir.map', 'style','popup', ...
    'position',ui_right(h10,CDX,70), ...
    'backgroundc',[.7 .7 .7], ...
    'callback',@skui_showwhat, 'tag','showwhat',...
    'value',skui_data{f}.showwhat,...
    'tooltipstring','Select whether to show the original image, the image with background removed, or the directionality map.');

h9 = ui_push('Measure',@skui_5x5,ui_right(h8,CDX,70),[1 .7 .6],'Measure and store dimensions of selected rectangle. (Drag rectangle first.)');
h = ui_push('Excl. all',@skui_excludeall,ui_above(h9,RDY2),h9,'Exclude selected region for worm finding in all images. (Drag rectangle first.)');
h = ui_push('Exclude',@skui_excluderegion,ui_above(h,RDY),h9,'Exclude selected region for worm finding in this image only. (Drag rectangle first.)');

h10 = ui_push('About',@skui_about,ui_right(h9,CDX*1,70),[.5 1 1],...
    'Show information about this program.','userdata','always');

h5 = ui_push('Make AVI',@skui_avi,ui_above(h10,RDY2),[1 .7 1],'Create AVI movie of results.');
h = ui_push('Animate',@skui_movie,ui_above(h5,5),h5,'Repeatedly cycle through image stack. Stop by clicking again, or by pressing Escape.');


set(f,'name','Wormfinder','numbertitle','off');

skui_data{f}.imax4=nan;

t_start = cputime;
if nargin>=1
  skui_load(f,ifn);
else
  skui_enableui(f);
end

if nargin>=2
  t_loaded = cputime;
  skui_detectall(nan,f);
  t_detected = cputime;
  skui_save(f,ofn);
  t_saved = cputime;
  close(f);
  
  fprintf(1,'Loading:    %.3f s\n',t_loaded-t_start);
  fprintf(1,'Detection:  %.3f s for %i frames\n',t_detected-t_loaded,skui_data{f}.N);
  fprintf(1,'Time/frame: %.3f s\n',(t_detected-t_loaded)/skui_data{f}.N);
  fprintf(1,'Saving:     %.3f s\n',t_saved-t_detected);
else
  skui_replot(f);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_saveres(h,x)
global skui_data
f=gcbf;
if skui_data{f}.N==0
  errordlg('Nothing to save.',...
	'Wormfinder','modal');
  return
end  
if isfield(skui_data{f},'ifnbase')
  prt=strtoks(skui_data{f}.ifnbase,'/');
  if length(prt)>=2
    bb=[prt{end-1} '-skeleton.mat'];
  else
    bb='';
  end
else
  bb='';
end
  [fn,pn]=uiputfile({'*.mat', 'Matlab files'},'Save results');
  if isequal(fn,0) | isequal(pn,0)
    ofn='';
  else
    ofn=[pn filesep fn];
  end

if ~isempty(ofn)
  skui_saveresult(f,ofn)
end

function skui_save(f,ofn)
global skui_data

if skui_data{f}.N==0
  errordlg('Nothing to save.',...
	'Wormfinder','modal');
  return
end  
skui_saveresult(f,ofn)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_saveresult(f,ofn)
global skui_data

skui_align(f);

ifnbase=skui_data{f}.ifnbase;
xxx=skui_data{f}.xxx;
yyy=skui_data{f}.yyy;
wei=skui_data{f}.wei;
thresh=skui_data{f}.thr;
scale=skui_data{f}.scale;
scalerect=skui_data{f}.scalerect;
im0=uint8(skui_data{f}.im0);
vsn='1.0.1';
sig_perp = scale(1,:);
sig_long = scale(2,:);
lambda = scale(3,:)*4;
if isfield(skui_data{f},'headpos')
  headpos=skui_data{f}.headpos;
else
  headpos=[];
end
if isfield(skui_data{f},'tailpos')
  tailpos=skui_data{f}.tailpos;
else
  tailpos=[];
end


save(ofn,'ifnbase','xxx','yyy','wei','thresh','scale',...
    'im0','scalerect','vsn','sig_perp','sig_long','lambda',...
    'headpos','tailpos');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_load(f,ifn)
if strcmp(ifn(end-3:end),'.mat')
  skui_loadresult(f,ifn);
  return
end

if exist(ifn,'dir')
  dd=dir(ifn);
  ok=0;
  for n=1:length(dd)
    try
      ii=imfinfo([ifn filesep dd(n).name]);
      ifn = [ifn filesep dd(n).name];
      ok=1;
      break
    catch
      ;
    end
  end
  if ~ok
    errordlg(sprintf('No images found in folder "%s". Cannot use.',...
	ifn),...
	'Wormfinder','modal');
    return
  end
end

try
  ii=imfinfo(ifn);
  imok=1;
catch
  imok=0;
end    
try
  ii=aviinfo(ifn);
  avok=1;
catch
  avok=0;
end

if ~imok & ~avok
  errordlg('Unknown file type. Wormfinder can load images or .mat.',...
      'Wormfinder','modal');
  return
end

z=find(ifn=='.');
if isempty(z)
  ifnbase=ifn;
  extn='';
else
  ifnbase=ifn(1:z(end)-1);
  extn=ifn(z(end)+1:end);
end

if imok
  n=0;
  while ifnbase(end)>='0' & ifnbase(end)<='9'
    ifnbase=ifnbase(1:end-1);
    n=n+1;
  end
  if floor(log10(atoi(ifn(length(ifnbase)+1:end))))+1 < n
    str = sprintf('%%0%ii',n);
  elseif n>0
    str = '%i';
  else
    str = '';
  end
  ifnbase = [ifnbase str '.' extn];
  ifnbase = strrep(ifnbase,'\\','\');
  ifnbase = strrep(ifnbase,'\\','\');
  ifnbase = strrep(ifnbase,'\\','\');
  ifnbase = strrep(ifnbase,'\','\\');
  if ~exist(sprintf(ifnbase,1),'file')
    ifnbase = ifn;
  end
end
skui_loadseries(f,ifnbase,[],avok);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_loadmat(h,x)
f=gcbf;
  [fn,pn]=uigetfile({ '*.mat','Matlab files'}, ...
      'Select a -skeleton.mat file to load');
  if isequal(fn,0) | isequal(pn,0)
    ifn='';
  else
    ifn=[pn filesep fn];
  end
if ~isempty(ifn)
  skui_load(f,ifn);
end

  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_loadpng(h,x)

f=gcbf;
  [fn,pn]=uigetfile(...
      { '*.png;*.gif;*.jpg;*.tif;*.tiff;*.jpeg','Image files'; ...
        '*.avi','AVI movies' }, ...
      'Select a movie, an image, or any one from a series');
  if isequal(fn,0) | isequal(pn,0)
    ifn='';
  else
    ifn=[pn filesep fn];
  end
if ~isempty(ifn)
  skui_load(f,ifn);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_loadseries(f,ifnbase,im0,avok)
global skui_data

skui_busy(f);

if nargin<3
  im0=[];
end
if nargin<4
  try
    avin=aviinfo(ifnbase);
    avok=1;
  catch
    avok=0;
  end
end

skui_data{f}.ifnbase = ifnbase;

if avok
  skui_data{f}.imgs = wf_loadavi(ifnbase, uir_on(f,'deint'));
else
  if any(ifnbase=='%')
    skui_data{f}.imgs = loadimseq(ifnbase, uir_on(f,'deint'));
  else
    skui_data{f}.imgs = loadimseq(ifnbase, 0);
  end
end
skui_data{f}.Y = size(skui_data{f}.imgs,1);
skui_data{f}.X = size(skui_data{f}.imgs,2);
skui_data{f}.N = size(skui_data{f}.imgs,3) * (numel(skui_data{f}.imgs)>0);
skui_data{f}.headpos = [];
skui_data{f}.tailpos = [];

if skui_data{f}.N > 1
  fprintf(1,'Calculating background...             \n');
  
  id0=find(skui_data{f}.ifnbase == '%' );
  id1=find(skui_data{f}.ifnbase == 'i' );
  mfn=[];
  if ~isempty(id0)
    id0=id0(1);
    id1=id1(id1>id0);
    if ~isempty(id1)
      id1=id1(1);
      mfn = [skui_data{f}.ifnbase(1:id0-1) 'median' skui_data{f}.ifnbase(id1+1:end)];
    end
  end
  
  if exist(mfn,'file');
    skui_data{f}.im0=imread(mfn); 
    skui_data{f}.im0=mean(double(skui_data{f}.im0),3);
  else
    skui_data{f}.im0 = [];%max(skui_data{f}.imgs,[],3);%median(skui_data{f}.imgs,3);
  end
else
  fprintf(1,'\n');
  skui_data{f}.im0 = [];
  uir_on(f,'cleanbg',0);
end

skui_data{f}.xxx = zeros(100,skui_data{f}.N) + nan;
skui_data{f}.yyy = zeros(100,skui_data{f}.N) + nan;
skui_data{f}.wei = zeros(100,skui_data{f}.N) + nan;
skui_data{f}.thr = zeros(1,skui_data{f}.N) + nan;
skui_data{f}.scale = zeros(3,skui_data{f}.N) + nan;
skui_data{f}.scalerect=[];
skui_data{f}.imax4=nan;
skui_data{f}.excludes = cell(skui_data{f}.N,1);

skui_data{f}.frame=1;

set(f,'name',sprintf('Wormfinder: %s',ifnbase));

skui_enableui(f);
skui_replot(f);

skui_free(f);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_loadresult(f,ifn)
global skui_data

inp = load(ifn);

if isempty(skui_data) | ~isfield(skui_data{f},'ifnbase') | ...
      ~strcmp(skui_data{f}.ifnbase,inp.ifnbase)
  if isfield(inp,'im0')
    skui_loadseries(f,inp.ifnbase,double(inp.im0));
  else
    skui_loadseries(f,inp.ifnbase);
  end
end

skui_data{f}.xxx = inp.xxx;
skui_data{f}.yyy = inp.yyy;
skui_data{f}.wei = inp.wei;

if isfield(inp,'headpos')
  skui_data{f}.headpos=inp.headpos;
else
  skui_data{f}.headpos=[];
end
if isfield(inp,'tailpos')
  skui_data{f}.tailpos=inp.tailpos;
else
  skui_data{f}.tailpos=[];
end

if isfield(inp,'thresh')
  skui_data{f}.thr = inp.thresh;
elseif isfield(inp,'thr')
  skui_data{f}.thr = inp.thr;
else  
  skui_data{f}.thr = zeros(1,size(inp.yyy,2))+nan;
end
if isfield(inp,'scale')
  skui_data{f}.scale = inp.scale;
  if size(inp.scale,1)==2
     skui_data{f}.scale(3,:) = skui_data{f}.scale(1,:)*.6;
   end
else
  skui_data{f}.scale = zeros(3,size(inp.yyy,2))+nan;
end
%skui_data{f}.sigperp = skui_data{f}.scale(1);
%skui_data{f}.siglong = skui_data{f}.scale(2);
%skui_data{f}.lambda = 4*skui_data{f}.scale(3);
set(findobj(f,'tag','sigperp'),'string',sprintf('%.1f',skui_data{f}.scale(1)));
set(findobj(f,'tag','siglong'),'string',sprintf('%.1f',skui_data{f}.scale(2)));
set(findobj(f,'tag','lambda'),'string',sprintf('%.1f',skui_data{f}.scale(3)*4));

if isfield(inp,'scalerect')
  skui_data{f}.scalerect = inp.scalerect;
end

if isfield(inp,'excludes')
  skui_data{f}.excludes = inp.excludes;
else
  skui_data{f}.excludes = cell(skui_data{f}.N,1);
end

skui_enableui(f);
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_replot(f)
global skui_data
figure(f);

if skui_data{f}.N==0
  cla;
  return
end

n=skui_data{f}.frame;

cla
newval = get(findobj(f,'tag','showwhat'),'value');
skui_data{f}.showwhat = newval;

switch newval
  case 2 % Cleaned
    skui_ensure_im0(f);
    h=imagesc(double(skui_data{f}.imgs(:,:,n)) - skui_data{f}.im0);
    colormap(gray); 
  case 3 % Dir.map
      skui_busy(f);
    if skui_data{f}.imax4 ~= skui_data{f}.frame
      im = double(skui_data{f}.imgs(:,:,skui_data{f}.frame));
      if uir_on(f,'cleanbg') & skui_data{f}.N>1
	skui_ensure_im0(f);
	im=im - double(skui_data{f}.im0);
      end
      %im = skui_doexclude(im,f,skui_data{f}.frame,mean(im(:)));
      [args,thr,scl] = skui_args(f);
      skui_data{f}.shownscale = [scl(1) scl(2) scl(3)*4];
      [imax,dirs] = directimg2(im,args{:});
      imax = skui_doexclude(imax,f,skui_data{f}.frame);
      skui_data{f}.imax = imax;
      skui_data{f}.dirs = dirs;
      skui_data{f}.imax4 = skui_data{f}.frame;
    end
    h=plotmaxdir(skui_data{f}.imax,-2*skui_data{f}.dirs+pi,1,.5);
    skui_free(f);
  otherwise % Image
    h=imagesc(skui_data{f}.imgs(:,:,n));
    colormap(gray); 
end
caxis('auto');
axis tight;
K=size(skui_data{f}.excludes{n},1);
hh=zeros(K,1);
for k=1:K
  rect = skui_data{f}.excludes{n}(k,:);
  hh(k) = rectangle(...
      'position',[rect(1) rect(3) rect(2)+1-rect(1) rect(4)+1-rect(3)],...
      'edgec','none','facec',[1 .5 .5]);
end
set([h;hh],'buttondownfcn',@skui_butdown);
skui_data{f}.rect=[];

hold on

if any(~isnan(skui_data{f}.xxx(:,n)))
  plot(skui_data{f}.xxx(:,n),skui_data{f}.yyy(:,n),'r-');
  plot(skui_data{f}.xxx(1,n),skui_data{f}.yyy(1,n),'r.');
end

if isfield(skui_data{f},'headpos')
  if size(skui_data{f}.headpos,1)>=n
    plot(skui_data{f}.headpos(n,1),skui_data{f}.headpos(n,2),'g.');
  end
end
  
if isfield(skui_data{f},'tailpos')
  if size(skui_data{f}.tailpos,1)>=n
    plot(skui_data{f}.tailpos(n,1),skui_data{f}.tailpos(n,2),'c.');
  end
end
  


drawnow

set(findobj(f,'tag','frameno'),...
    'string',sprintf('%03i/%03i',skui_data{f}.frame,skui_data{f}.N));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_detect(h,x)
global skui_data
if isnan(h)
  f=x;
else
  f=gcbf;
end
skui_busy(f);

n=skui_data{f}.frame;

[args, thr, scale] = skui_args(f);
if skui_data{f}.imax4==n
  [xx,yy,im] = findworm({skui_data{f}.imax,skui_data{f}.dirs},args{:});
else
  im = double(skui_data{f}.imgs(:,:,n));
  if uir_on(f,'cleanbg') & skui_data{f}.N>1
    skui_ensure_im0(f);
    im0=double(skui_data{f}.im0);
    [xx,yy,im] = findworm(im-im0, args{:});
  else
    [xx,yy,im] = findworm(im, args{:});
  end
end

skui_data{f}.xxx(:,n) = xx;
skui_data{f}.yyy(:,n) = yy;
skui_data{f}.wei(:,n) = im;
skui_data{f}.thr(:,n) = thr;
skui_data{f}.scale(:,n) = scale;

skui_replot(f);
fprintf(1,'                                                      \r');
skui_free(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_detectall(h,x)
global skui_data

if isnan(h)
  f=x;
  h=[];
else
  f=gcbf;
end

skui_busy(f);

if strcmp(get(h,'fontw'),'bold')
  % Already working -> Abort
  set(f,'currentchar',char(27));
  skui_free(f);
  return
end

set(f,'currentchar','.');
set(h,'fontw','bold');
for n=1:skui_data{f}.N
  if any(isnan(skui_data{f}.xxx(:,n)));
    skui_data{f}.frame = n;
    fprintf(1,'Detecting worm in frame %i/%i...\n',n,skui_data{f}.N);
    skui_detect(nan,f);
    skui_align(f);
    %title 'Press Escape to abort detection'
    if get(f,'currentchar')+0==27
      break;
    end
  end
end
set(h,'fontw','normal');

skui_free(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_prev(h,x)
global skui_data
f=gcbf;

skui_data{f}.frame = max(skui_data{f}.frame-1,1);
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_first(h,x)
global skui_data
f=gcbf;

skui_data{f}.frame = 1;
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_next(h,x)
global skui_data
f=gcbf;

skui_data{f}.frame = min(skui_data{f}.frame+1,skui_data{f}.N);
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_last(h,x)
global skui_data
f=gcbf;

skui_data{f}.frame = skui_data{f}.N;
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_frameno(h,x)
global skui_data
f=gcbf;

n = atoi(get(h,'string'));
if n>0 & n<=skui_data{f}.N
  skui_data{f}.frame = n;
  skui_replot(f);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_butdown(h,x)
global skui_data
f=gcbf;
xy0=get(gca,'currentpoint'); xy0=xy0(1,1:2);
but=get(gcf,'selectiontype');
if strcmp(but,'normal')
  skui_dragrect(h,x);
elseif strcmp(but,'extend')
  skui_data{f}.headpos(skui_data{f}.frame,:) = xy0;
  skui_replot(f);
elseif strcmp(but,'alt')
  skui_data{f}.tailpos(skui_data{f}.frame,:) = xy0;
  skui_replot(f);
end

function skui_dragrect(h,x)
global skui_data
f=gcbf;
xy0=get(gca,'currentpoint'); xy0=xy0(1,1:2);
rbbox;
xy1=get(gca,'currentpoint'); xy1=xy1(1,1:2);

xx=sort([xy0(1) xy1(1)]);
yy=sort([xy0(2) xy1(2)]);
skui_data{f}.rect=round([xx yy]);
hh=findobj(f,'type','rectangle','tag','dragrect');
delete(hh);
rectangle('position',[xx(1) yy(1) diff(xx)+1 diff(yy)+1],'edgec','r',...
    'tag','dragrect');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_excluderegion(h,x)
global skui_data
f=gcbf;
if isfield(skui_data{f},'rect') & ~isempty(skui_data{f}.rect)
  skui_addexclude(skui_data{f}.frame,skui_data{f}.rect,f);
end
skui_data{f}.imax4=nan;
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_excludeall(h,x)
global skui_data
f=gcbf;
if isfield(skui_data{f},'rect') & ~isempty(skui_data{f}.rect)
  for n=1:skui_data{f}.N
    skui_addexclude(n,skui_data{f}.rect,f);
  end
end
skui_data{f}.imax4=nan;
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_addexclude(n,rect,f)
global skui_data
%skui_data{f}.imgs(rect(3):rect(4),rect(1):rect(2),n) = ...
%      mean(mean(skui_data{f}.imgs(:,:,n)));
skui_data{f}.excludes{n}(end+1,:)=rect(:)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function img = skui_doexclude(img,f,n,val)
global skui_data
K=size(skui_data{f}.excludes{n},1);
if nargin<4
  val=0;
end
for k=1:K
  rect = skui_data{f}.excludes{n}(k,:);
  img(rect(3):rect(4),rect(1):rect(2))=val;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_5x5(h,x)
global skui_data
f=gcbf;

if isfield(skui_data{f},'rect') & ~isempty(skui_data{f}.rect)
  skui_data{f}.scalerect = skui_data{f}.rect;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_movie(h,x)
global skui_data
f=gcbf;

if strcmp(get(h,'fontw'),'bold')
  % Already playing movie -> Abort
  set(f,'currentchar',char(27));
  return
end

skui_align(f);

n0 = skui_data{f}.frame;

set(f,'currentchar','.');
set(h,'fontw','bold');
ok=1;
n=n0;
while ok
  n=n+1;
  if n>skui_data{f}.N
    n=1;
  end
  skui_data{f}.frame = n;
  skui_replot(f);
  %title 'Press Escape to stop animation'
  if get(f,'currentchar')+0==27
    ok=0;
    break;
  end
  pause(0.1);
end
set(h,'fontw','normal');

%skui_data{f}.frame = n0;
skui_replot(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_avi(h,x)
global skui_data
f=gcbf;

skui_busy(f);

skui_align(f);

if ischar(h)
  ofn=h;
else
  if isfield(skui_data{f},'ifnbase')
    prt=strtoks(skui_data{f}.ifnbase,'/');
    if length(prt)>=2
      bb=[prt{end-1} '-skeleton.avi'];
    else
      bb='';
    end
  else
    bb='';
  end
    [fn,pn]=uiputfile({'*.avi', 'AVI movies'},'Save AVI movie');
    if isequal(fn,0) | isequal(pn,0)
      ofn='';
    else
      ofn=[pn filesep fn];
    end
    
  if isempty(ofn)
    return;
  end
end

n0 = skui_data{f}.frame;

boxh = msgbox('Please wait, creating AVI file','Wormfinder');

mv = avifile(ofn);
for n=1:skui_data{f}.N
  skui_data{f}.frame = n;
  skui_replot(f);
  if 0
    text(638,2,sprintf('%s %.3f s',skui_data{f}.ifnbase,(n-1)/60),...
	'horizontala','right','verticala','top','color','r','fontsi',12);
  end
  fr=getframe(gca);
  mv = addframe(mv,fr);
end
mv = close(mv);


try
  delete(boxh);
end

skui_data{f}.frame = n0;
skui_replot(f);

skui_free(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_align(f)
global skui_data
[skui_data{f}.xxx,skui_data{f}.yyy,skui_data{f}.wei] = ...
    alignworms(skui_data{f}.xxx,skui_data{f}.yyy,skui_data{f}.wei);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_showwhat(h,x)
global skui_data
f=gcbf;
oldval = skui_data{f}.showwhat;
newval = get(h,'value');
if oldval~=newval
  skui_data{f}.showwhat = newval;
  skui_replot(f);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_lambda(h,x)
global skui_data
f=gcbf;
n = skui_data{f}.frame;
oldval = skui_data{f}.shownscale(3);
newval = ui_getnum('lambda');
if isempty(newval)
  newval = ui_getnum('sigperp')*2.5;
end
if oldval~=newval
  skui_invalidateimax(h,x);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_sigperp(h,x)
global skui_data
f=gcbf;
n = skui_data{f}.frame;
oldval = skui_data{f}.shownscale(1);
newval = ui_getnum('sigperp');
if oldval~=newval
  skui_invalidateimax(h,x);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_siglong(h,x)
global skui_data
f=gcbf;
n = skui_data{f}.frame;
oldval = skui_data{f}.shownscale(2);
newval = ui_getnum('siglong');
if isempty(newval)
  newval = ui_getnum('sigperp');
end
if oldval~=newval
  skui_invalidateimax(h,x);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_lightdark(h,x)
h1=findobj(gcbf,'tag','ondark');
h2=findobj(gcbf,'tag','onlight');
if h==h1
  uir_on(gcbf,'onlight',0);
else
  uir_on(gcbf,'ondark',0);
end
skui_invalidate_im0(gcbf)
skui_invalidateimax(h,x)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_invalidate_im0(f)
global skui_data
skui_data{f}.im0 = [];
if skui_data{f}.showwhat>1
  skui_replot(f);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_ensure_im0(f)
global skui_data
if isempty(skui_data{f}.im0)
  ondark = uir_on(f,'ondark');
  if ondark
    skui_data{f}.im0 = min(skui_data{f}.imgs,[],3);
  else
    skui_data{f}.im0 = max(skui_data{f}.imgs,[],3);
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_invalidateimax(h,x)
global skui_data
f=gcbf;
skui_data{f}.imax4 = nan;
if skui_data{f}.showwhat==3
  skui_replot(f);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_busy(f)
global skui_data
if ~isempty(f) & ~isnan(f)
  if isfield(skui_data{f},'busy')
    skui_data{f}.busy = skui_data{f}.busy + 1;
  else
    skui_data{f}.busy=1;
  end
  if skui_data{f}.busy==1
    set(f,'pointer','watch');
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_free(f)
global skui_data
if ~isempty(f) & ~isnan(f)
  skui_data{f}.busy = skui_data{f}.busy - 1;
  if skui_data{f}.busy==0
    set(f,'pointer','arrow');
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_enableui(f)
global skui_data
if skui_data{f}.N>0
  val='on';
else
  val='off';
end
hh=findobj(f,'type','uicontrol');
set(hh,'enable',val);
hh=findobj(f,'userdata','always');
set(hh,'enable','on');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function skui_about(h,x)
w=which('wormfinder');
if isempty(w)
  ww='';
else
  z=find(w==filesep);
  if isempty(z)
    ;
  else
    w=w(1:z(end));
  end
  ww=[ 'An electronic version, named "wormfinder.pdf", may be found in ', ...
       '"' w '".' ];
end

h=msgbox({'Wormfinder 1.0.0 -- (C) 2007-2009  Daniel Wagenaar'
    ''
  [ 'Wormfinder comes ABSOLUTELY NO WARRANTY; ' ...
    'for details, click "Details" below. ' ...
    'This is free software, and you are welcome to redistribute it ' ...
    'under certain conditions; again, click "Details" for details.' ]
    ''
  [ 'Basic help is provided by the tooltips that should pop up when ' ...
    'you move the mouse pointer over any button.']
    ''
%  [ 'For more information, please consult the user manual. ' ww]
%    ''
  [ 'The algorithm employed by this software is described in ' ...
    'Wagenaar, D. A., 2009, "Automated video analysis of worm movements."' ...
    '']
    },'Wormfinder');

hok=findobj(h,'type','uicontrol');
if ~isempty(hok)
  set(h,'unit','pix');
  xywh0=get(h,'position');
  set(hok,'unit','pix');
  for h1=hok(:)'
    xywh=get(h1,'position');
    xywh(1) = xywh0(3) - xywh(3) - 10;
    set(h1,'position',xywh);
  end
end

uicontrol(h,'style','pushbutton', 'position', [10 10 50 25],...
    'string','Details','callback',@skui_details);

function skui_details(h,x)
h=msgbox({'Wormfinder 1.0.0 -- (C) 2007-2009  Daniel Wagenaar'
    ''
  [ 'Wormfinder is free software; you can redistribute it and/or modify ' ...
    'it under the terms of the GNU General Public License as published by ' ...
    'the Free Software Foundation; either version 2 of the License, or ' ...
    '(at your option) any later version.' ]
    ''
  [ 'Wormfinder is distributed in the hope that it will be useful, ' ...
    'but WITHOUT ANY WARRANTY; without even the implied warranty of ' ...
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the ' ...
    'GNU General Public License for more details. ' ]
    ''
  [ 'You should have received a copy of the GNU General Public License ' ...
    'along with this program; if not, write to the Free Software ' ...
    'Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.']
    },'Wormfinder');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [args, thr, scale] = skui_args(f)
global skui_data
n = skui_data{f}.frame;
scl1 = str2double(get(findobj(f,'tag','sigperp'),'string'));
scl2 = str2double(get(findobj(f,'tag','siglong'),'string'));
if isnan(scl2)
  scl2 = scl1;
end
scl3 = str2double(get(findobj(f,'tag','lambda'),'string'));
if isnan(scl3)
  scl3 = scl1 * 2.5;
end
l4 = scl3/4;

thr = ui_getnum('thresh');
if isempty(thr) | thr<0 | thr>1
  thr=.3;
end
% New version, as of 8/10/07
args={ 'thresh',thr, ...
       'interp',100, ...
       'pwr',ui_getnum('edgepower'), ...
       'sigperp',scl1, ...
       'lambda4',l4, ...
       'istep',scl2/2, ...
       'irange',scl1/2, ...
       'siglong',scl2, ...
       'onlight',uir_on(f,'onlight'), ...
       'ondark',uir_on(f,'ondark'), ...
       'softmax',uir_on(f,'softmax'),...
       'exclude',skui_data{f}.excludes{n},...
       };

scale=[scl1;scl2;l4];
